@@ -35,6 +35,7 @@
 #include <dlfcn.h>
 #include <limits.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -195,13 +196,13 @@ trap_path(const char *path)
     return buf;
 }
 
-static dev_t
-get_rdev(const char *nodename)
+static bool
+get_rdev_maj_min(const char *nodename, uint32_t *major, uint32_t *minor)
 {
     static char buf[PATH_MAX];
     static char link[PATH_MAX];
     int name_offset;
-    int major, minor, orig_errno;
+    int orig_errno;
 
     name_offset = snprintf(buf, sizeof(buf), "%s/dev/.node/", getenv("UMOCKDEV_DIR"));
     buf[sizeof(buf) - 1] = 0;
@@ -217,15 +218,26 @@ get_rdev(const char *nodename)
     if (readlink(buf, link, sizeof(link)) < 0) {
 	DBG(DBG_PATH, "get_rdev %s: cannot read link %s: %m\n", nodename, buf);
 	errno = orig_errno;
-	return (dev_t) 0;
+	return false;
     }
     errno = orig_errno;
-    if (sscanf(link, "%i:%i", &major, &minor) != 2) {
+    if (sscanf(link, "%u:%u", major, minor) != 2) {
 	DBG(DBG_PATH, "get_rdev %s: cannot decode major/minor from '%s'\n", nodename, link);
-	return (dev_t) 0;
+	return false;
     }
-    DBG(DBG_PATH, "get_rdev %s: got major/minor %i:%i\n", nodename, major, minor);
-    return makedev(major, minor);
+    DBG(DBG_PATH, "get_rdev %s: got major/minor %u:%u\n", nodename, *major, *minor);
+    return true;
+}
+
+static dev_t
+get_rdev(const char *nodename)
+{
+    unsigned major, minor;
+
+    if (get_rdev_maj_min(nodename, &major, &minor))
+	return makedev(major, minor);
+    else
+	return (dev_t) 0;
 }
 
 static dev_t
@@ -1164,6 +1176,20 @@ rettype name(const char *path, arg2t arg2, arg3t arg3, arg4t arg4) \
     return r;							\
 }
 
+#define STAT_ADJUST_MODE \
+    if (ret == 0 && p != path && strncmp(path, "/dev/", 5) == 0			\
+	&& is_emulated_device(p, st->st_mode)) {				\
+	st->st_mode &= ~S_IFREG;						\
+	if (st->st_mode & S_ISVTX) {						\
+            st->st_mode = S_IFBLK | (st->st_mode & ~S_IFMT);			\
+	    DBG(DBG_PATH, "  %s is an emulated block device\n", path);		\
+	} else {								\
+            st->st_mode = S_IFCHR | (st->st_mode & ~S_IFMT);			\
+	    DBG(DBG_PATH, "  %s is an emulated char device\n", path);		\
+	}									\
+	st->st_rdev = get_rdev(path + 5);					\
+    }										\
+
 /* wrapper template for stat family; note that we abuse the sticky bit in
  * the emulated /dev to indicate a block device (the sticky bit has no
  * real functionality for device nodes) */
@@ -1182,18 +1208,27 @@ int prefix ## stat ## suffix (const char *path, struct stat ## suffix *st) \
     DBG(DBG_PATH, "testbed wrapped " #prefix "stat" #suffix "(%s) -> %s\n", path, p);	\
     ret = _ ## prefix ## stat ## suffix(p, st);					\
     TRAP_PATH_UNLOCK;								\
-    if (ret == 0 && p != path && strncmp(path, "/dev/", 5) == 0			\
-	&& is_emulated_device(p, st->st_mode)) {				\
-	st->st_mode &= ~S_IFREG;						\
-	if (st->st_mode &  S_ISVTX) {						\
-	    st->st_mode &= ~S_ISVTX; st->st_mode |= S_IFBLK;			\
-	    DBG(DBG_PATH, "  %s is an emulated block device\n", path);		\
-	} else {								\
-	    st->st_mode |= S_IFCHR;						\
-	    DBG(DBG_PATH, "  %s is an emulated char device\n", path);		\
-	}									\
-	st->st_rdev = get_rdev(path + 5);					\
+    STAT_ADJUST_MODE;                                                           \
+    return ret;									\
+}
+
+/* wrapper template for fstatat family */
+#define WRAP_FSTATAT(prefix, suffix) \
+int prefix ## fstatat ## suffix (int dirfd, const char *path, struct stat ## suffix *st, int flags) \
+{ \
+    const char *p;								\
+    libc_func(prefix ## fstatat ## suffix, int, int, const char*, struct stat ## suffix *, int); \
+    int ret;									\
+    TRAP_PATH_LOCK;								\
+    p = trap_path(path);							\
+    if (p == NULL) {								\
+	TRAP_PATH_UNLOCK;							\
+	return -1;								\
     }										\
+    DBG(DBG_PATH, "testbed wrapped " #prefix "fstatat" #suffix "(%s) -> %s\n", path, p); \
+    ret = _ ## prefix ## fstatat ## suffix(dirfd, p, st, flags);		\
+    TRAP_PATH_UNLOCK;								\
+    STAT_ADJUST_MODE;                                                           \
     return ret;									\
 }
 
@@ -1218,21 +1253,31 @@ int prefix ## stat ## suffix (int ver, const char *path, struct stat ## suffix *
     DBG(DBG_PATH, "testbed wrapped " #prefix "stat" #suffix "(%s) -> %s\n", path, p);	\
     ret = _ ## prefix ## stat ## suffix(ver, p, st);				\
     TRAP_PATH_UNLOCK;								\
-    if (ret == 0 && p != path && strncmp(path, "/dev/", 5) == 0			\
-	&& is_emulated_device(p, st->st_mode)) {				\
-	st->st_mode &= ~S_IFREG;						\
-	if (st->st_mode &  S_ISVTX) {						\
-	    st->st_mode &= ~S_ISVTX; st->st_mode |= S_IFBLK;			\
-	    DBG(DBG_PATH, "  %s is an emulated block device\n", path);		\
-	} else {								\
-	    st->st_mode |= S_IFCHR;						\
-	    DBG(DBG_PATH, "  %s is an emulated char device\n", path);		\
-	}									\
-	st->st_rdev = get_rdev(path + 5);					\
-    }										\
+    STAT_ADJUST_MODE;                                                           \
     return ret;									\
 }
 
+/* wrapper template for __fxstatat family */
+#define WRAP_VERFSTATAT(prefix, suffix) \
+int prefix ## fxstatat ## suffix (int ver, int dirfd, const char *path, struct stat ## suffix *st, int flags) \
+{ \
+    const char *p;								\
+    libc_func(prefix ## fxstatat ## suffix, int, int, int, const char*, struct stat ## suffix *, int); \
+    int ret;									\
+    TRAP_PATH_LOCK;								\
+    p = trap_path(path);							\
+    if (p == NULL) {								\
+	TRAP_PATH_UNLOCK;							\
+	return -1;								\
+    }										\
+    DBG(DBG_PATH, "testbed wrapped " #prefix "fxstatat" #suffix "(%s) -> %s\n", path, p); \
+    ret = _ ## prefix ## fxstatat ## suffix(ver, dirfd, p, st, flags);		\
+    TRAP_PATH_UNLOCK;								\
+    STAT_ADJUST_MODE;                                                           \
+    return ret;									\
+}
+
+
 /* wrapper template for open family */
 #define WRAP_OPEN(prefix, suffix) \
 int prefix ## open ## suffix (const char *path, int flags, ...)	    \
@@ -1319,10 +1364,12 @@ WRAP_2ARGS(int, -1, chmod, mode_t);
 WRAP_2ARGS(int, -1, access, int);
 WRAP_STAT(,);
 WRAP_STAT(l,);
+WRAP_FSTATAT(,);
 
 #ifdef __GLIBC__
 WRAP_STAT(,64);
 WRAP_STAT(l,64);
+WRAP_FSTATAT(,64);
 WRAP_FOPEN(,64);
 #endif
 
@@ -1336,6 +1383,48 @@ WRAP_VERSTAT(__x,);
 WRAP_VERSTAT(__x, 64);
 WRAP_VERSTAT(__lx,);
 WRAP_VERSTAT(__lx, 64);
+
+#ifdef HAVE_FXSTATAT
+WRAP_VERFSTATAT(__,);
+WRAP_VERFSTATAT(__,64);
+#endif
+
+int statx(int dirfd, const char *pathname, int flags, unsigned mask, struct statx * stx)
+{
+    const char *p;
+    libc_func(statx, int, int, const char *, int, unsigned, struct statx *);
+    int r;
+
+    TRAP_PATH_LOCK;
+    p = trap_path(pathname);
+    DBG(DBG_PATH, "testbed wrapped statx (%s) -> %s\n", pathname, p ?: "NULL");
+    if (p == NULL)
+        r = -1;
+    else
+        r = _statx(dirfd, p, flags, mask, stx);
+    TRAP_PATH_UNLOCK;
+
+    if (r == 0 && p != pathname && strncmp(pathname, "/dev/", 5) == 0
+            && is_emulated_device(p, stx->stx_mode)) {
+        if (stx->stx_mode & S_ISVTX) {
+            stx->stx_mode = S_IFBLK | (stx->stx_mode & ~S_IFMT);
+            DBG(DBG_PATH, "  %s is an emulated block device (statx)\n", pathname);
+        } else {
+            stx->stx_mode = S_IFCHR | (stx->stx_mode & ~S_IFMT);
+            DBG(DBG_PATH, "  %s is an emulated char device (statx)\n", pathname);
+        }
+	unsigned maj, min;
+	if (get_rdev_maj_min(pathname + 5, &maj, &min)) {
+	    stx->stx_rdev_major = maj;
+	    stx->stx_rdev_minor = min;
+	} else {
+	    stx->stx_rdev_major = stx->stx_rdev_minor = 0;
+	}
+
+    }
+    return r;
+}
+
 #endif
 
 int __open_2(const char *path, int flags);
