@@ -33,6 +33,8 @@
 #include "utils.h"
 #include "uevent_sender.h"
 
+#define UEVENT_BUFSIZE 16384
+
 struct _uevent_sender {
     char *rootpath;
     char socket_glob[PATH_MAX];
@@ -226,12 +228,11 @@ append_property(char *array, size_t size, size_t offset, const char *name, const
 void
 uevent_sender_send(uevent_sender * sender, const char *devpath, const char *action, const char *properties)
 {
-    char buffer[1024];
+    char buffer[UEVENT_BUFSIZE];
     size_t buffer_len = 0;
     struct msghdr smsg;
     struct iovec iov[2];
     const char *subsystem;
-    const char *devname;
     const char *devtype;
     char seqnumstr[20];
     struct udev_device *device;
@@ -247,7 +248,6 @@ uevent_sender_send(uevent_sender * sender, const char *devpath, const char *acti
     subsystem = udev_device_get_subsystem(device);
     assert(subsystem != NULL);
 
-    devname = udev_device_get_devnode(device);
     devtype = udev_device_get_devtype(device);
 
     /* build NUL-terminated property array */
@@ -256,10 +256,6 @@ uevent_sender_send(uevent_sender * sender, const char *devpath, const char *acti
     buffer_len += append_property(buffer, sizeof buffer, buffer_len, "SUBSYSTEM=", subsystem);
     snprintf(seqnumstr, sizeof(seqnumstr), "%llu", seqnum++);
     buffer_len += append_property(buffer, sizeof buffer, buffer_len, "SEQNUM=", seqnumstr);
-    if (devname)
-        buffer_len += append_property(buffer, sizeof buffer, buffer_len, "DEVNAME=", devname);
-    if (devtype)
-        buffer_len += append_property(buffer, sizeof buffer, buffer_len, "DEVTYPE=", devtype);
 
     /* append udevd (userland) properties, replace \n with \0 */
     /* FIXME: more sensible API */
